
#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>
#include "ui_mainwindow.h"

#include <QGraphicsView>
#include <QGraphicsSceneMouseEvent>
#include <QGraphicsTextItem>
#include <QGraphicsLineItem>
#include <QGraphicsRectItem>
#include <QGraphicsItemGroup>
//#include <QImage>
#include <QProcess>
#include <QtMultimedia/QMediaPlayer>
#include <QAudioOutput>
#include <QTimer>

#include "complexe.h"


struct ENR_FFT
{
    double amplitude; // 8 octets
    uint32_t t; // 4 octets
    uint8_t num; // 1 octet
    uint8_t midi; //1 octet
// total 4+1+1+8 = 14 octets
};


// à propos des notes : PRINCIPE
// les notes ont une fréquence qui n'est pas directement enregistrée ici mais est représenté par le numéro MIDI.
// elles ont un emplacement qui est enregistré et qui permet de connaitre l'instant de son jeu, connaissant la grille.
// pour la grille temporelle, voir 'tracer_grille_MESURES()'
// la variable 'duree' pourrait être calculée en fonction des précédentes, mais le fait de l'avoir ici
// permet de visualiser une toute partie de la partition, voir une note unique, en dehors du contexte
struct NOTE
{
    uint16_t numero;  // 2 octets
    uint8_t midi;     // 1 octet
    uint16_t n_barreT; // 2 octets; numéro de la barre temporelle (T, 1/20s)sur laquelle est posée la note
    int16_t n_barreM; // 2 octets; num de barre M sur laquelle.. (accepte les valeurs négatives)
// total 7 octets
};



/* FFT */

class MainWindow : public QMainWindow, public Ui::MainWindow
{
    Q_OBJECT
    
public:
    explicit MainWindow(QWidget *parent = 0);
    ~MainWindow();

protected:

    void mousePressEvent(QMouseEvent *event);
    void contextMenuEvent(QContextMenuEvent *event); // clic de droite


private slots:
    void Tic1();
    void Tic2();
    void save_fichier_ini();
    void load_fichier_ini();
    void choix_dossier_de_travail();
    void open_fichier_wav();

    void on_Bt_load_wav_clicked();
    void ajustement_gain();
    void load_fichier_FRQ();
    int save_fichier_FRQ(QString date_i);
    void load_fichier_NOTES();
    int save_fichier_NOTES(QString date_i);

    void decode_entete();
    void garnis_temp_data(qint32 offset);

    void parametrages();
    void init_TW_1();
    void init_TAB_FRQ();
    void init_TAB_lst_notes();
    void init_TW_type_M();
    void init_Autres();

    void Etiquette(int x, int dx, int y, QString s, QString coul_txt, QString coul_fond, QString coul_cadre , QGraphicsItemGroup *calque_i);

    void afficher_titre_calque(QString titre, int x, int y, QGraphicsItemGroup *calque_i);
    void tracer_graduations_signal();
    void tracer_graduations_FFT(); // sur onglet 'Source wav'
    void tracer_signal_complet(); // sur onglet 'Source wav'
    void tracer_gradu_temporelle_signal_entree(); // sur onglet 'Source wav'
    void tracer_signal_a_convertir(); // sur onglet 'Source wav'
    void tracer_graduation_TAB_NOTES(); // midi ;  en colonne à gauche + noms des notes
    void tracer_echelle_temps_TAB_NOTES(); //en secondes  (scene4 sur l'onglet NOTES)
    void tracer_grille_M();
    void tracer_grille_T();
    double calcul_pas_grille();
    //double calcul_pas_grille_M();
    double calcul_x_barre(int n_barre);
    int calcul_tempo();
    //double calcul_x_barre_M(int n_barre);
    void trace_1_barre_M(double xi, QString couleur_i);
    void tracer_1enr(ENR_FFT enr_i);
    void detection_notes_auto(); // notes secondaires (affi par petits cercles colorés)
    void retracer_TAB_FRQ();
    void maj_TAB_NOTES();

    void encadrement_signal(int x, int dx);
    void encadrement_note(int n_midi);
    void encadre_midi(int n_midi);
    int calcul_num_midi(double x);
    void tracer_segments_FFT();

    void complete_case(int row, int column, QString s1, QString c1, QString c2, bool bold1, QTableWidget *QTI);
    void affiche_liste_notes();
    void trace_liste_notes(); //dessine des cercles colorés (couleur midi) sur la grille

    void calcul_histogramme_durees();
    void affiche_histogramme_durees();

    void calcul_histogramme_midi();
    void affiche_histogramme_midi();

    void calcul_histogramme_degre(); // degrés de la gamme chromatique = 1..12
    void affiche_histogramme_degre();

    void affiche_histogramme();

    QString denombre_diezes_a_la_cle();
    int test_if_note_in_liste(uint16_t n_barre, int midi);
    void affi_txt_en_couleur(int midi_i, QLineEdit *lineEdit_i);

    int num_barre(double xi);
    void ajout_note(int midi, int n_barreT, int n_barreM);
    void deplace_note(int n, int d);
    void supprime_note(uint16_t n);
    uint16_t calcul_n_barreM_note(uint16_t n); // barre M la plus proche de la note posée sur une barre T
    uint16_t calcul_n_barreT_note(uint16_t n); // barre T la plus proche de la note posée sur une barre M

    void remplir_tableau_echantillons_signal();
    void calcul_compression();
    void bit_reverse_tableau_X();

    int calcul_y_note(int midi); // en pixels sur la grille
    int calcul_hauteur_note(int mid);
    void calcul_tableau_W();
    void calcul_FFT();
    void calcul_ofsset();
    void traitement_signal();
    void analyse_tout();
    QString nom_note(int mid);
    QString nom_note_GB(int mid);
    QString nom_octave_GB(int mid);

    void numerotation_liste_notes();
    void tri_liste_notes_par_num_barre();
    void suppression_notes_invalides();
    void suppression_mesures();

    void play_note(int midi);// sortie audio
    int joue_1_note_de_la_liste();
    void surbrille_note(uint16_t n);
    void trace_time_line();

    void effacer_calque(QGraphicsScene *scene_i, QGraphicsItemGroup *calque_i);

    void on_spinBox_1_valueChanged(int arg1);
    void on_spinBox_2_valueChanged(int arg1);
    void on_spinBox_3_valueChanged(int arg1);
    void on_spinBox_4_valueChanged(int arg1);
    void on_spinBox_5_valueChanged(int arg1);
    void on_spinBox_6_valueChanged();
    void on_spinBox_7_valueChanged(int arg1);
    void on_pushButton_8_clicked();
    void on_doubleSpinBox_1_valueChanged();
    void on_Bt_RAZ_clicked();
    void on_Bt_scan_auto_clicked();
    void on_pushButton_2_clicked();
    void on_Btn_52_clicked();
    void on_Btn_94_clicked();
    void on_pushButton_3_clicked();
    void on_Bt_close_entete_clicked();
     void on_pushButton_6_clicked();
    void on_Bt_efface_clicked();
    void on_pushButton_5_clicked();
    void on_checkBox_rapide_stateChanged(int arg1);
    void on_doubleSpinBox_seuil_valueChanged(double arg1);
    void on_checkBox_norm_clicked();
    void on_Bt_jouer_tout_clicked();

    void mode_grille_T();
    void on_Bt_toggle_grille_T_clicked();
    void on_Bt_toggle_visu_notes_clicked();
    void mode_grille_M();
    void on_Bt_toggle_grille_M_clicked();
    void on_Bt_save_notes_clicked();
    void on_Bt_load_notes_clicked();
    void on_Bt_joue_passage_clicked();
    void on_Bt_joue_wav_clicked();
    void on_Btn_RAZ_notes_clicked();
    void on_Bt_stop_clicked();
    void on_Bt_stop_2_clicked();
    void on_Bt_mode_R_clicked();
    void on_Bt_mode_W_clicked();
    void on_Bt_toggle_visu_freq_clicked();
    void on_Bt_efface_2_clicked();
    void on_Bt_copy_nt_T_to_M_clicked();
    void on_Bt_copy_nt_M_to_T_clicked();
    void on_Bt_joue_wav_2_clicked();
    void on_Bt_choix_current_dir_clicked();
    void on_pushButton_9_clicked();
    void on_spinBox_8_textChanged();
    void on_doubleSpinBox_t0_valueChanged();
    void on_spinBox_offset_valueChanged();
    void on_spinBox_zoom_2_valueChanged(int arg1);
    void on_spinBox_d_barres_valueChanged();
    void on_Bt_load_FREQ_clicked();
    void on_Bt_decale_notes_L_clicked();
    void on_Bt_decale_notes_R_clicked();
    void on_tableWidget_type_M_cellClicked(int row, int column);
    void on_checkBox_flitrer_clicked();
    void on_Bt_open_DIR_clicked();
    void on_Bt_goto_clicked();
    void on_doubleSpinBox_x0_valueChanged(double arg1);
    void on_doubleSpinBox_T0_valueChanged(double arg1);
    void on_Bt_test1_clicked();
    void on_Bt_div2_clicked();
    void on_pushButton_clicked();
    void on_Bt_next_clicked();
    void on_Bt_prev_clicked();
    void on_Bt_debut_clicked();
    void on_Bt_fin_clicked();
    void on_Bt_durees_div2_clicked();
    void on_Bt_durees_mult2_clicked();
    void on_bt_test_clicked();
    void on_Bt_detection_notes_clicked();
    void on_Bt_toggle_visu_notes_auto_clicked();
    void on_Bt_m10_clicked();
    void on_Bt_p10_clicked();
    void on_doubleSpinBox_gain_valueChanged(double arg1);
    void on_Bt_test2_clicked();
    void on_Bt_open_DIR_2_clicked();
    void on_spinBox_nb_barres_valueChanged(int arg1);
    void on_spinBox_barre_zero_valueChanged(int arg1);
    void on_pushButton_10_clicked();
    void on_Bt_mode_E_clicked();
    void on_Bt_RAZ_general_clicked();
    void on_spinBox_x0a_valueChanged(int arg1);
    void on_doubleSpinBox_x0b_valueChanged(double arg1);

    void on_spinBox_dxa_valueChanged(int arg1);

    void on_Bt_deplace_L_clicked();
    void on_Bt_deplace_R_clicked();

    void on_doubleSpinBox_dxb_valueChanged(double arg1);

    void on_pushButton_4_clicked();
    void on_pushButton_11_clicked();


    void on_radioButton_D_clicked();

    void on_radioButton_M_clicked();

    void on_radioButton_H_clicked();

private:

    QPoint Zoom_point2_;

  //  QImage image1;

    QGraphicsScene *scene1; // sur l'onglet 'Source' - Trace FFT
    QGraphicsItemGroup *calque_reticule1;
    QGraphicsItemGroup *calque_trace_signal1;
    QGraphicsItemGroup *calque_trace_FFT;
    QGraphicsItemGroup *calque_curseur;
    QGraphicsItemGroup *calque_encadrement1;
    QGraphicsItemGroup *calque_lignes_F_zero;

    QGraphicsScene *scene2; // sur l'onglet 'Source' - Signal d'entrée
    QGraphicsItemGroup *calque_reticule2;
    QGraphicsItemGroup *calque_encadrement2;
    QGraphicsItemGroup *calque_trace_signal2;

    QGraphicsScene *scene3;
    QGraphicsItemGroup *calque_gradu_TAB_FRQ;

    QGraphicsScene *scene4; // sur l'onglet 'NOTES'
    QGraphicsItemGroup *calque_TAB_FRQ;

    QGraphicsScene *scene5; // sur l'onglet 'NOTES'
    QGraphicsItemGroup *calque_histogramme;

    QGraphicsItemGroup *calque_grille_T;
    QGraphicsItemGroup *calque_grille_M;
    QGraphicsItemGroup *calque_echelle_temporelle;
    QGraphicsItemGroup *calque_notes_manuelles;
    QGraphicsItemGroup *calque_notes_auto;
    QGraphicsItemGroup *calque_notes_jouee;

    QGraphicsLineItem *ligne1;
    QGraphicsLineItem *segment_trace;
    QGraphicsLineItem *segment_trace2;
    QGraphicsRectItem *rectangle1;
    QGraphicsRectItem *rectangle2;
    QGraphicsEllipseItem *ellipse1;
    QGraphicsRectItem *rect1;
    QGraphicsRectItem *rect2;
    QGraphicsRectItem *rect3;

    QGraphicsTextItem *GraphicsTextItem;

    QMediaPlayer *player1;
    QMediaPlayer *player2;
    QMediaPlayer *player3;
    QMediaPlayer *player4;
    QMediaPlayer *player5;

    QAudioOutput  *audioOutput1;
    QAudioOutput  *audioOutput2;
    QAudioOutput  *audioOutput3;
    QAudioOutput  *audioOutput4;
    QAudioOutput  *audioOutput5;

    QTimer *Timer1; // pour analyse auto à faible vitesse
    QTimer *Timer2; // pour rejouer les notes détectées

};



#endif // MAINWINDOW_H
